/**********************************************************************\

File:		menus.c

Purpose:	This module handles menu selections.

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program in a file named "GNU General Public License".
If not, write to the Free Software Foundation, 675 Mass Ave,
Cambridge, MA 02139, USA.

\**********************************************************************/

#include "graphics.h"
#include "menus.h"
#include "help.h"
#include "environment.h"
#include "sounds.h"
#include "error.h"
#include "launch.h"
#include "file interface.h"
#include "ls load-save.h"
#include "ls endgame.h"
#include "ls find.h"
#include "ls shift.h"
#include "ls.h"
#include "debinhex dispatch.h"
#include "file management.h"
#include "program globals.h"

extern	long		menuDisable : 0x0b54;

Boolean			gMenuEnabled;
MenuHandle		gAppleMenu;
MenuHandle		gFileMenu;
MenuHandle		gEditMenu;
MenuHandle		gOptionsMenu;
MenuHandle		gBoardMenu;
MenuHandle		gPlayMenu;

enum
{
	appleMenu = 400, fileMenu, editMenu, optionsMenu, playMenu,
	
	aboutItem = 1, aboutMSGItem, helpPointerItem,
	
	newItem = 1, openItem, closeItem, saveItem, saveAsItem, deBinHexItem = 7,
		launchItem=9, forceQuitItem = 11, quitItem,
	
	undoItem = 1, cutItem = 3, copyItem, pasteItem, clearItem, selectAllItem,
	
	soundToggle = 1, options_unused1, board5, board6, board7, board8, options_unused2,
		hideItem,
	
	findItem=1, play_unused1, shiftLeftItem, shiftRightItem, shiftUpItem, shiftDownItem
};

/*-----------------------------------------------------------------------------------*/
/* internal stuff for menus.c                                                        */

static	void HandleAppleMenu(short menuItem);
static	void HandleFileMenu(short menuItem);
static	void HandleEditMenu(short menuItem);
static	void HandleHelpMenu(void);
static	void HandleOptionsMenu(short menuItem);
static	void HandlePlayMenu(short menuItem);

Boolean InitTheMenus(void)
{
	Handle		MBARHandle;
	
	if ((MBARHandle=GetNewMBar(400))==0L)		/* sez which menus are in menu bar. */
		return FALSE;
	SetMenuBar(MBARHandle);						/* set this to be THE menu bar to use. */
	
	if ((gAppleMenu=GetMHandle(appleMenu))==0L)	/* GetNewMBar also got menu handles of */
		return FALSE;
	if ((gFileMenu=GetMHandle(fileMenu))==0L)	/* every menu it includes, so just */
		return FALSE;
	if ((gEditMenu=GetMHandle(editMenu))==0L)	/* grab these handles and assign them */
		return FALSE;
	if ((gOptionsMenu=GetMHandle(optionsMenu))==0L)
		return FALSE;
	if ((gPlayMenu=GetMHandle(playMenu))==0L)
		return FALSE;
	
	if (gSystemSevenOrLater)
	{
		AppendMenu(gOptionsMenu, "\p-");
		AppendMenu(gOptionsMenu, "\pHide/1");
	}
	
	AddResMenu(gAppleMenu, 'DRVR');				/* adds control panels to apple menu */
	
	AdjustMenus();								/* dim/enable/check/mark menus/items */
	DrawMenuBar();								/* draws the actual menu bar */
	
	return TRUE;
}

void AdjustMenus(void)
{
	WindowPeek		theWindow;
	short			kind;
	short			i;
	
	if (gInProgress)
	{
		DisableItem(gAppleMenu, aboutItem);
		DisableItem(gAppleMenu, aboutMSGItem);
		DisableItem(gAppleMenu, helpPointerItem);
		if (gSystemSevenOrLater)
		{
			DisableItem(gFileMenu, newItem);
			DisableItem(gFileMenu, openItem);
			DisableItem(gFileMenu, closeItem);
			DisableItem(gFileMenu, deBinHexItem);
			DisableItem(gFileMenu, launchItem);
			DisableItem(gFileMenu, quitItem);
		}
		else DisableItem(gFileMenu, 0);
		
		DisableItem(gEditMenu, 0);
		DisableItem(gOptionsMenu, 0);
		DisableItem(gPlayMenu, 0);
	}
	else
	{
		EnableItem(gAppleMenu, aboutItem);
		EnableItem(gAppleMenu, aboutMSGItem);
		EnableItem(gAppleMenu, helpPointerItem);
		if (gSystemSevenOrLater)
		{
			EnableItem(gFileMenu, newItem);
			EnableItem(gFileMenu, openItem);
			EnableItem(gFileMenu, closeItem);
			EnableItem(gFileMenu, deBinHexItem);
			EnableItem(gFileMenu, launchItem);
			EnableItem(gFileMenu, quitItem);
		}
		else EnableItem(gFileMenu, 0);
		EnableItem(gEditMenu, 0);
		EnableItem(gOptionsMenu, 0);
		
		theWindow = (WindowPeek)FrontWindow();
		kind = theWindow ? theWindow->windowKind : 0;
		
		if (kind < 0)
		{
			EnableItem(gEditMenu, undoItem);
			EnableItem(gEditMenu, cutItem);
			EnableItem(gEditMenu, copyItem);
			EnableItem(gEditMenu, pasteItem);
			EnableItem(gEditMenu, clearItem);
		}
		else
		{
			if ((theWindow!=0L) && ((WindowPtr)theWindow==GetIndWindowGrafPtr(kMainWindow)) &&
				(gNumMoves>0) && (!ShowingFindQQ()) && (!ShowingEndGameQQ()))
				EnableItem(gEditMenu, undoItem);
			else
				DisableItem(gEditMenu, undoItem);
			DisableItem(gEditMenu, cutItem);
			DisableItem(gEditMenu, copyItem);
			DisableItem(gEditMenu, pasteItem);
			DisableItem(gEditMenu, clearItem);
		}
		
		if (theWindow)
			EnableItem(gFileMenu, closeItem);
		else
			DisableItem(gFileMenu, closeItem);
		
		if (GetIndWindowGrafPtr(kMainWindow))
		{
			DisableItem(gFileMenu, newItem);
			DisableItem(gFileMenu, openItem);
			EnableItem(gFileMenu, saveItem);
			EnableItem(gFileMenu, saveAsItem);
			DisableItem(gOptionsMenu, board5);
			DisableItem(gOptionsMenu, board6);
			DisableItem(gOptionsMenu, board7);
			DisableItem(gOptionsMenu, board8);
			if (ShowingEndGameQQ() || ShowingFindQQ())
				DisableItem(gPlayMenu, 0);
			else
				EnableItem(gPlayMenu, 0);
		}
		else
		{
			EnableItem(gFileMenu, newItem);
			EnableItem(gFileMenu, openItem);
			DisableItem(gFileMenu, saveItem);
			DisableItem(gFileMenu, saveAsItem);
			EnableItem(gOptionsMenu, board5);
			EnableItem(gOptionsMenu, board6);
			EnableItem(gOptionsMenu, board7);
			EnableItem(gOptionsMenu, board8);
			DisableItem(gPlayMenu, 0);
		}
		
		if (gSystemSevenOrLater)
			EnableItem(gFileMenu, forceQuitItem);
		else
			DisableItem(gFileMenu, forceQuitItem);
		
		if (gSoundAvailable)
			EnableItem(gOptionsMenu, soundToggle);
		else
			DisableItem(gOptionsMenu, soundToggle);
	}
	CheckItem(gOptionsMenu, soundToggle, gSoundToggle&&gSoundAvailable);
	for (i=0; i<=MAX_ROWS-5; i++)
		CheckItem(gOptionsMenu, board5+i, (gNumRows==(i+5)));
}

void HandleMenu(long mSelect)
{
	short			menuID = HiWord(mSelect);
	short			menuItem = LoWord(mSelect);
	
	if (menuID==0)
	{
		menuID=HiWord(menuDisable);
		menuItem=LoWord(menuDisable);
		gMenuEnabled=FALSE;
	}
	else gMenuEnabled=TRUE;
	menuDisable=0L;

	switch (menuID)
	{
		case appleMenu:
			HandleAppleMenu(menuItem);
			break;
		case fileMenu:
			HandleFileMenu(menuItem);
			break;	
		case editMenu:
			HandleEditMenu(menuItem);
			break;
		case optionsMenu:
			HandleOptionsMenu(menuItem);
			break;
		case playMenu:
			HandlePlayMenu(menuItem);
			break;
	}
}

void DoTheCloseThing(WindowPeek theWindow)
/* a standard close procedure, called when "close" is chosen from File menu and when
   a window is closed through its close box */
{
	Boolean			gotone;
	short			i;
	short			kind;
	
	if (theWindow==0L)
		return;
	
	kind = theWindow ? theWindow->windowKind : 0;
	if (kind<0)		/* DA window or other system window */
		CloseDeskAcc(kind);
	else
	{
		gotone=FALSE;
		/* see if it's one of ours */
		for (i=0; (i<NUM_WINDOWS) && (!gotone); i++)
			gotone=((WindowPtr)theWindow==GetIndWindowGrafPtr(i));
		
		if (gotone)		/* if it's one of ours...  see graphics.c */
			CloseTheIndWindow(i-1);			/* this may return FALSE = not closed */
		else
			DisposeWindow((WindowPtr)theWindow);	/* not one of ours, so just close it */
	
		AdjustMenus();	/* may affect which menu items or menus are available, etc */
	}
}

void HandleAppleMenu(short menuItem)
{
	GrafPtr		savePort;
	Str255		name;
	
	switch (menuItem)
	{
		case aboutItem:
			if (gMenuEnabled)
				OpenTheIndWindow(kAbout);
			else DoSound(sound_fluff, TRUE);
			break;
		case aboutMSGItem:
			if (gMenuEnabled)
				OpenTheIndWindow(kAboutMSG);
			else DoSound(sound_fluff, TRUE);
			break;
		case helpPointerItem:
			if (gMenuEnabled)
				HandleHelpMenu();
			else DoSound(sound_fluff, TRUE);
			break;
		default:
			if (menuItem > helpPointerItem+1)
			{
				GetPort(&savePort);
				GetItem(gAppleMenu, menuItem, name);
				OpenDeskAcc(name);
				SetPort(savePort);
			}
			break;
	}
}

void HandleFileMenu(short menuItem)
{
	WindowPtr			theWindow;
	short				i;
	Boolean				gotone;
	
	switch (menuItem)
	{
		case newItem:
			if (gMenuEnabled)
				NewGame();
			else DoSound(sound_fluff, TRUE);
			break;
		case openItem:
			if (gMenuEnabled)
				LoadSaveDispatch(TRUE, FALSE);
			else DoSound(sound_fluff, TRUE);
			break;
		case closeItem:
			if (gMenuEnabled)
				DoTheCloseThing((WindowPeek)FrontWindow());
			else DoSound(sound_fluff, TRUE);
			break;
		case saveItem:
			if (gMenuEnabled)
				LoadSaveDispatch(FALSE, TRUE);
			else DoSound(sound_fluff, TRUE);
			break;
		case saveAsItem:
			if (gMenuEnabled)
				LoadSaveDispatch(FALSE, FALSE);
			else DoSound(sound_fluff, TRUE);
			break;
		case deBinHexItem:
			if (gMenuEnabled)
			{
				if (GetSourceFile(&inputFS, TRUE, FALSE))
					HandleError(DeBinHexDispatch(), FALSE);
			}
			else DoSound(sound_fluff, TRUE);
			break;
		case launchItem:
			if (gMenuEnabled)
			{
				LaunchDispatch();
			}
			else DoSound(sound_fluff, TRUE);
			break;
		case forceQuitItem:
			if (gMenuEnabled)
				SysError(0x4e22);
			else DoSound(sound_fluff, TRUE);
			break;
		case quitItem:
			if (gMenuEnabled)
				gDone = TRUE;
			else DoSound(sound_fluff, TRUE);
			break;
	}
}

void HandleEditMenu(short menuItem)
{
	if ((gMenuEnabled) && (menuItem!=2) && (menuItem>0))
	{
		if (gFrontWindowIsOurs)
		{
			switch (menuItem)
			{
				case undoItem:		CallIndDispatchProc(gFrontWindowIndex, kUndo, 0L);	break;
				case cutItem:		CallIndDispatchProc(gFrontWindowIndex, kCut, 0L);	break;
				case copyItem:		CallIndDispatchProc(gFrontWindowIndex, kCopy, 0L);	break;
				case pasteItem:		CallIndDispatchProc(gFrontWindowIndex, kPaste, 0L);	break;
				case clearItem:		CallIndDispatchProc(gFrontWindowIndex, kClear, 0L);	break;
				case selectAllItem:	CallIndDispatchProc(gFrontWindowIndex, kSelectAll, 0L);	break;
			}
		}
		else SystemEdit(menuItem-1);
	}
	else if ((menuItem!=2) && (menuItem>0))
		DoSound(sound_fluff, TRUE);
}

void HandleOptionsMenu(short menuItem)
{
	switch (menuItem)
	{
		case soundToggle:
			if (gMenuEnabled)
			{
				gSoundToggle=!gSoundToggle;
				DoSound(sound_on, TRUE);
			}
			break;
		case board5:
		case board6:
		case board7:
		case board8:
			if (gMenuEnabled)
			{
				gNumRows=gNumColumns=menuItem-board5+5;
			}
			else DoSound(sound_fluff, TRUE);
			break;
		case hideItem:
			if (gMenuEnabled)
			{
				MenuKey(0);
			}
			else DoSound(sound_fluff, TRUE);
			break;
	}
}

void HandleHelpMenu(void)
{
	OpenTheIndWindow(kHelp);
}

void HandlePlayMenu(short menuItem)
{
	switch (menuItem)
	{
		case findItem:
			if (gMenuEnabled)
			{
				FindDispatch(GetIndWindowDataHandle(kMainWindow));
			}
			else DoSound(sound_fluff, TRUE);
			break;
		case shiftLeftItem:
			if (gMenuEnabled)
			{
				ShiftLeftDispatch(GetIndWindowDataHandle(kMainWindow));
			}
			else DoSound(sound_fluff, TRUE);
			break;
		case shiftRightItem:
			if (gMenuEnabled)
			{
				ShiftRightDispatch(GetIndWindowDataHandle(kMainWindow));
			}
			else DoSound(sound_fluff, TRUE);
			break;
		case shiftUpItem:
			if (gMenuEnabled)
			{
				ShiftUpDispatch(GetIndWindowDataHandle(kMainWindow));
			}
			else DoSound(sound_fluff, TRUE);
			break;
		case shiftDownItem:
			if (gMenuEnabled)
			{
				ShiftDownDispatch(GetIndWindowDataHandle(kMainWindow));
			}
			else DoSound(sound_fluff, TRUE);
			break;
	}
}
